@@ -22,6 +22,7 @@
 
 #include <msgpack.h>
 #include <ruby.h>
+#include <ruby/debug.h>
 
 #ifndef RUBY_VM
 #include <env.h>
@@ -51,6 +52,11 @@
 #endif
 
 
+// The SUN_LEN macro is not available on Android
+#ifndef SUN_LEN
+#define SUN_LEN(ptr) ((size_t) (((struct sockaddr_un *) 0)->sun_path) + strlen((ptr)->sun_path))
+#endif
+
 static uint64_t
 ru_utime_usec()
 {
@@ -688,7 +694,7 @@ rbtracer_add(char *query, bool is_slow)
       *idx = NULL,
       *method = NULL;
 
-    if (NULL != (idx = rindex(query, '.'))) {
+    if (NULL != (idx = strrchr(query, '.'))) {
       klass_begin = 0;
       klass_end = idx - query;
       is_singleton = true;
@@ -700,7 +706,7 @@ rbtracer_add(char *query, bool is_slow)
 
       method = idx+1;
 
-    } else if (NULL != (idx = rindex(query, '#'))) {
+    } else if (NULL != (idx = strrchr(query, '#'))) {
       klass_begin = 0;
       klass_end = idx - query;
       is_singleton = false;
@@ -862,6 +868,20 @@ msgq_setup()
 #endif
 }
 
+static VALUE rbtrace_module;
+
+static VALUE
+eval_inspect(VALUE rb_code) {
+  return rb_funcall(rbtrace_module, rb_intern("eval_and_inspect"), 1, rb_code);
+}
+
+static VALUE
+rescue_inspect(VALUE arg) {
+  VALUE exception = rb_errinfo(); /* get last exception */
+  rb_set_errinfo(Qnil);
+  return rb_funcall(exception, rb_intern("inspect"), 0);
+}
+
 static void
 rbtrace__process_event(msgpack_object cmd)
 {
@@ -1012,8 +1032,10 @@ rbtrace__process_event(msgpack_object cmd)
     strncpy(query, str.ptr, str.size);
     query[str.size] = 0;
 
-    snprintf(code, BUF_SIZE+150, "(begin; %s; rescue Exception => e; e; end).inspect", query);
-    val = rb_eval_string_protect(code, 0);
+    VALUE rb_code;
+    rb_code = rb_str_new2(query);
+
+    val = rb_rescue(eval_inspect, rb_code, rescue_inspect, Qnil);
 
     if (TYPE(val) == T_STRING) {
       rbtrace__send_event(1,
@@ -1021,7 +1043,6 @@ rbtrace__process_event(msgpack_object cmd)
         's', RSTRING_PTR(val)
       );
     }
-
   }
 }
 
@@ -1102,9 +1123,40 @@ signal_handler_wrapper(VALUE arg, VALUE ctx)
 }
 #endif
 
+static VALUE
+send_write(VALUE klass, VALUE val) {
+  if (TYPE(val) == T_STRING) {
+    rbtrace__send_event(1,
+      "write",
+      's', RSTRING_PTR(val)
+    );
+  }
+
+  return Qnil;
+}
+
 void
 Init_rbtrace()
 {
+  rbtrace_module = rb_define_module("RBTrace");
+  VALUE output = rb_define_module_under(rbtrace_module, "OUT");
+
+  rb_define_singleton_method(output, "write", send_write, 1);
+
+  rb_eval_string(
+    "module RBTrace\n"
+    "  def self.eval_context\n"
+    "    @eval_context ||= binding\n"
+    "  end\n"
+
+    "  def self.eval_and_inspect(code)\n"
+    "    t = Thread.new { Thread.current[:output] = eval_context.eval(code).inspect }\n"
+    "    t.join\n"
+    "    t[:output]\n"
+    "  end\n"
+    "end\n"
+  );
+
   // hook into the gc
   gc_hook = Data_Wrap_Struct(rb_cObject, rbtrace_gc_mark, NULL, NULL);
   rb_global_variable(&gc_hook);
