@@ -12,7 +12,7 @@ class RBTraceCLI
   #
   # Returns nothing.
   def self.check_msgmnb
-    if File.exists?(msgmnb = "/proc/sys/kernel/msgmnb")
+    if File.exist?(msgmnb = "/proc/sys/kernel/msgmnb")
       curr = File.read(msgmnb).to_i
       max = 1024*1024
       cmd = "sysctl kernel.msgmnb=#{max}"
@@ -34,7 +34,7 @@ class RBTraceCLI
   # Returns nothing.
   def self.cleanup_queues
     if (pids = `ps ax -o pid`.split("\n").map{ |p| p.strip.to_i }).any?
-      ipcs = `ipcs -q`.split("\n").grep(/^(q|0x)/).map{ |line| line[/(0x[a-f0-9]+)/,1] }
+      ipcs = `ipcs -q`.split("\n").grep(/^(q|0x)/).map{ |line| line[/(0x[a-f0-9]+)/,1] }.compact
       ipcs.each do |ipci|
         next if ipci.match(/^0xf/)
 
@@ -273,7 +273,7 @@ EOS
         file = [
           config,
           File.expand_path("../../../tracers/#{config}.tracer", __FILE__)
-        ].find{ |f| File.exists?(f) }
+        ].find{ |f| File.exist?(f) }
 
         unless file
           parser.die :config, '(file does not exist)'
